// CustomCollection.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#import "c:\\Program Files\\Rational\\Rose\\rationalrose.tlb" no_namespace

int main(int argc, char* argv[])
{

	// Initialize the COM library.
	CoInitialize(NULL);

	CLSID RoseID;
	HRESULT hr;
	// Get the CLSID from the registry base
	// (HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Rose.Application\CLSID)
	hr = CLSIDFromProgID(L"Rose.Application", &RoseID);
	if(FAILED(hr))
	{ 
		CoUninitialize();
		return -1;
	}


	// Get the active Rose instance
	LPUNKNOWN lpUnk;
	hr = GetActiveObject(RoseID, NULL, (LPUNKNOWN *)&lpUnk);

	if(FAILED(hr))
	{
		CoUninitialize();
		return -2;
	}

	// Ask for an interface pointer
	LPDISPATCH lpDispatch;
	hr = lpUnk->QueryInterface(IID_IDispatch, (LPVOID FAR*)&lpDispatch);

	// Now that we have the interface pointer, we don't need the LPUNKNOWN pointer anymore.
	// Because this is a "basic" pointer (and not a smart pointer) we have to mannualy release the interface.
	lpUnk->Release();
	if(FAILED(hr))
	{
		CoUninitialize();
		return -3;
	}

	// Create an IRoseApplicationPtr from the interface pointer
	// Note: You cannot instanciate "IRose..." classes since they all are abstracts
	//       The "IRose...Ptr" classes are smart pointers to "IRose..." classes.
	//       They handles resource allocation and deallocation, via function calls
	//       to the IUnknown member functions: QueryInterface, AddRef, and Release.
	// Info: See help on _com_ptr_t or _COM_SMARTPTR_TYPEDEF or IUnknown 
	IRoseApplicationPtr roseApp(lpDispatch);

	// Begin a new block (curly braces) to allocate a new space in the system stack
	// so that the objects are guaranted to be destructed at the end of this block.
	// This is important because the smart pointers destructors calls Release() and
	// this has to be done before roseApp->Exit(); and CoUninitialize();

	{ 
		IRoseModelPtr theModel = roseApp->GetCurrentModel();
		IRoseClassCollectionPtr allClasses = theModel->GetAllClasses();
		// build a custom collection of classes containing the interface classes of
		// the current model
		// The instanciable classes are only those figuring in the registry.
		// (HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Rose.*)
		IRoseClassCollectionPtr interfaceClasses(L"Rose.ClassCollection");
		int iClass=1;
		printf("interfaceClasses:\n");
		for(iClass=1; iClass<=allClasses->GetCount(); iClass++)
		{
			IRoseClassPtr aClass = allClasses->GetAt(iClass);			
			if(aClass->GetStereotype() == _bstr_t("Interface"))
			{
				interfaceClasses->Add(aClass);
				printf("%s\n", (char *)aClass->GetName());
			}
		}

		// build a custom collection of classes containing the classes with
		// a name starting with I
		IRoseClassCollectionPtr IClasses(L"Rose.ClassCollection");
		printf("IClasses:\n");
		for(iClass=1; iClass<=allClasses->GetCount(); iClass++)
		{
			IRoseClassPtr aClass = allClasses->GetAt(iClass);			
			if(((char *)aClass->GetName())[0] == 'I')
			{
				IClasses->Add(aClass);
				printf("%s\n", (char *)aClass->GetName());
			}
		}


		// remove the classes having a name starting with I from the interface
		// collection
		for(iClass=1; iClass<=IClasses->GetCount(); iClass++)
		{
			IRoseClassPtr aClass = IClasses->GetAt(iClass);
			if(interfaceClasses->Exists(aClass))
				interfaceClasses->Remove(aClass);
		}

		// print the names of the classes in interfaceClasses
		printf("interfaceClasses - IClasses:\n");
		for(iClass=1; iClass<=interfaceClasses->GetCount(); iClass++)
		{
			IRoseClassPtr aClass = interfaceClasses->GetAt(iClass);
			printf("%s\n", (char *)aClass->GetName());
		}

	} 


	lpDispatch->Release();

	// Uninitialize the COM library.
	CoUninitialize();

	return 0;
}

